<?php

namespace Drupal\views_area_link\Plugin\views\area;

use Drupal\Component\Serialization\Exception\InvalidDataTypeException;
use Drupal\Component\Serialization\Yaml;
use Drupal\Component\Utility\Html;
use Drupal\Core\Access\AccessManagerInterface;
use Drupal\Core\Form\FormState;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Language\LanguageInterface;
use Drupal\Core\Language\LanguageManagerInterface;
use Drupal\Core\Routing\RedirectDestinationTrait;
use Drupal\Core\Url;
use Drupal\views\Plugin\views\area\TokenizeAreaPluginBase;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Base class for Link area handler plugins.
 */
abstract class LinkBase extends TokenizeAreaPluginBase {

  use RedirectDestinationTrait;

  /**
   * The language manager.
   *
   * @var \Drupal\Core\Language\LanguageManagerInterface
   */
  protected $languageManager;

  /**
   * The access manager service.
   *
   * @var \Drupal\Core\Access\AccessManagerInterface
   */
  protected $accessManager;

  /**
   * Constructs a new Entity instance.
   *
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin_id for the plugin instance.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\Core\Language\LanguageManagerInterface $language_manager
   *   The language manager.
   * @param \Drupal\Core\Access\AccessManagerInterface $access_manager
   *   The access manager.
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, LanguageManagerInterface $language_manager, AccessManagerInterface $access_manager) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->languageManager = $language_manager;
    $this->accessManager = $access_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('language_manager'),
      $container->get('access_manager')
    );
  }

  /**
   * {@inheritdoc}
   */
  protected function defineOptions() {
    $options = parent::defineOptions();

    $options['base_info'] = [
      'default' => [
        'link_text' => '',
        'destination' => TRUE,
        'prefix' => '',
        'suffix' => '',
        'output_as_action' => FALSE,
      ],
    ];
    $options['advanced_opts'] = [
      'default' => [
        'absolute' => FALSE,
        'fragment' => '',
        'query' => [],
        'rewrite_output' => '',
        'access_denied_content' => '',
        'language' => '**auto**',
      ],
    ];

    return $options;
  }

  /**
   * {@inheritdoc}
   */
  public function buildOptionsForm(&$form, FormStateInterface $form_state) {
    parent::buildOptionsForm($form, $form_state);

    $base_info = $this->options['base_info'];

    $form['base_info'] = ['#tree' => TRUE];
    $form['base_info']['link_text'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Link text'),
      '#default_value' => $base_info['link_text'],
      '#description' => $this->t('The text to use for the link. May use tokens.'),
      '#required' => TRUE,
    ];
    $form['base_info']['prefix'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Prefix'),
      '#default_value' => $base_info['prefix'],
      '#description' => $this->t('Any text to display before this link. You may include HTML and tokens.'),
    ];
    $form['base_info']['suffix'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Suffix'),
      '#default_value' => $base_info['suffix'],
      '#description' => $this->t('Any text to display after this link. You may include HTML and tokens.'),
    ];
    $form['base_info']['output_as_action'] = [
      '#title' => $this->t('Output as action'),
      '#type' => 'checkbox',
      '#default_value' => $base_info['output_as_action'],
      '#description' => $this->t('Outputs the link as an "action button", themed like the links in the "Primary admin actions" block.'),
    ];
    $form['base_info']['destination'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Include destination'),
      '#default_value' => $this->options['base_info']['destination'],
      '#description' => $this->t('Include a "destination" parameter in the link to return the user to the original view upon completing the link action.'),
    ];
    $form['advanced_opts'] = [
      '#type' => 'details',
      '#title' => $this->t('Advanced Options'),
      '#weight' => 50,
    ];
    $form['advanced_opts']['absolute'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Use absolute path'),
      '#default_value' => $this->options['advanced_opts']['absolute'],
      '#description' => $this->t('Whether to force the output to be an absolute link (beginning with http:). Useful for links that will be displayed outside the site, such as in an RSS feed.'),
    ];
    $form['advanced_opts']['fragment'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Anchor'),
      '#default_value' => $this->options['advanced_opts']['fragment'],
      '#description' => $this->t("A fragment identifier (named anchor) to append to the URL.  Do not include the leading '#' character. Tokens allowed"),
    ];
    $form['advanced_opts']['query'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Queries'),
      '#default_value' => $this->yamlEncode($this->options['advanced_opts']['query']),
      '#description' => $this->t('A YAML array of query key/value-pairs (without any URL-encoding) to append to the URL. Tokens allowed.'),
    ];
    $form['advanced_opts']['rewrite_output'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Rewrite output'),
      '#default_value' => $this->options['advanced_opts']['rewrite_output'],
      '#description' => $this->t('Use this to output whatever HTML you want with the link created usable via the token {{views_area_link}}. Global tokens available as well.'),
    ];
    $form['advanced_opts']['access_denied_content'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Content when access is denied'),
      '#default_value' => $this->options['advanced_opts']['access_denied_content'],
      '#description' => $this->t('Use this to output whatever you want to display when access to the link is denied to the user. Tokens allowed.'),
    ];
    $form['advanced_opts']['language'] = [
      '#type' => 'radios',
      '#title' => $this->t('Language'),
      '#default_value' => $this->options['advanced_opts']['language'],
      '#options' => ['**auto**' => $this->t('Current language')] + $this->listLanguages(
        LanguageInterface::STATE_ALL | LanguageInterface::STATE_SITE_DEFAULT,
        [$this->options['advanced_opts']['language']]
      ),
    ];

    if (!$this->languageManager->isMultilingual()) {
      $form['advanced_opts']['language']['#description'] = $this->t('This will not take effect until you turn on multilingual capabilities');
    }
  }

  /**
   * Converts PHP array into YAML array.
   *
   * @param array $data
   *   The PHP array to convert to YAML.
   *
   * @return string
   *   The YAML encoded version of the array.
   */
  protected function yamlEncode(array $data) {
    if ($data === []) {
      return '';
    }
    return Yaml::encode($data);
  }

  /**
   * {@inheritdoc}
   */
  public function validateOptionsForm(&$form, FormStateInterface $form_state) {
    $options = $form_state->getValue('options');

    $options['advanced_opts']['query'] = $this->decodeYamlOpt(
      $form['advanced_opts']['query'],
      $form_state,
      $options['advanced_opts']['query']
    );

    $form_state->setValue('options', $options);
  }

  /**
   * Decodes and validates inputed YAML arrays.
   *
   * @param array $element
   *   The form element, for setError usage.
   * @param \Drupal\Core\Form\FormState $form_state
   *   The current form state.
   * @param string $encoded_opt
   *   The YAML encoded array to validate/decode to PHP.
   *
   * @return array
   *   The PHP version of the array.
   */
  public function decodeYamlOpt($element, FormState $form_state, $encoded_opt) {
    if ($encoded_opt === '') {
      return [];
    }

    try {
      $decoded_opt = Yaml::decode($encoded_opt);
      if (is_array($decoded_opt)) {
        return $decoded_opt;
      }
      $form_state->setError($element, $this->t('Value must be an array.'));
    }
    catch (InvalidDataTypeException $e) {
      $form_state->setError($element, $this->t('YAML does not validate: @exception', [
        '@exception' => $e->getMessage(),
      ]));
    }
    return [];
  }

  /**
   * Takes an input \Drupal\Core\Url object and outputs it as needed.
   *
   * @param Drupal\Core\Url $url
   *   Standard Drupal URL object.
   *
   * @return array
   *   Render array containing the content of this area.
   */
  protected function renderUrl(Url $url) {
    $options = $url->getOptions();

    $options['absolute'] = $this->options['advanced_opts']['absolute'];
    $options['fragment'] = $this->options['advanced_opts']['fragment'];
    $options['query'] = $this->options['advanced_opts']['query'];

    if (!empty($this->options['base_info']['destination'])) {
      $options['query'] += \Drupal::destination()->getAsArray();
    }

    if ($this->languageManager->isMultilingual() && $this->options['advanced_opts']['language'] !== '**auto') {
      $options['language'] = $this->languageManager->getLanguage($this->options['advanced_opts']['language']);
    }

    $url->setOptions($options);

    if ($this->checkUrlAccess($url) == FALSE) {
      return [
        '#markup' => $this->sanitizeValue($this->tokenizeValue($this->options['advanced_opts']['access_denied_content'])),
      ];
    }

    $link_text = $this->tokenizeValue($this->options['base_info']['link_text']);

    $link = [
      '#type' => 'link',
      '#url' => $url,
      '#title' => strip_tags(Html::decodeEntities($link_text)),
    ];

    if (($prefix = $this->options['base_info']['prefix']) !== '') {
      $link['#prefix'] = $this->tokenizeValue($prefix);
    }
    if (($suffix = $this->options['base_info']['suffix']) !== '') {
      $link['#suffix'] = $this->tokenizeValue($suffix);
    }

    if ($this->options['base_info']['output_as_action']) {
      $link = $this->outputAsActionLink($link);
    }

    if ($this->options['advanced_opts']['rewrite_output'] !== '') {
      $rewritten_output = $this->viewsTokenReplace(
        $this->options['advanced_opts']['rewrite_output'],
        ['views_area_link' => $link]
      );
      return [
        '#markup' => $this->sanitizeValue($this->tokenizeValue($rewritten_output), 'xss_admin'),
      ];
    }

    return $link;
  }

  /**
   * Checks access to the link route.
   *
   * @param Drupal\Core\Url $url
   *   Standard Drupal URL object.
   *
   * @return bool
   *   Whether the current user has access to the URL.
   */
  protected function checkUrlAccess(Url $url) {
    if ($url->isRouted() == FALSE) {
      return TRUE;
    }
    return $this->accessManager->checkNamedRoute($url->getRouteName(), $url->getRouteParameters());
  }

  /**
   * Takes the generated link ant transforms it into a 'local action' link.
   *
   * @param array $link
   *   The link to transform.
   *
   * @return array
   *   The render array for the generated action link.
   */
  protected function outputAsActionLink(array $link) {
    $action_link = [
      '#theme' => 'menu_local_action',
      '#link' => [
        'url' => $link['#url'],
        'title' => $link['#title'],
      ],
    ];

    $prefix = isset($link['prefix']) ? $link['prefix'] : '';
    $suffix = isset($link['suffix']) ? $link['suffix'] : '';
    $action_link['#prefix'] = $prefix . '<ul class="action-links">';
    $action_link['#suffix'] = '</ul>' . $suffix;

    return $action_link;
  }

}
