<?php

namespace Drupal\views_area_link\Plugin\views\area;

use Drupal\Core\Access\AccessManagerInterface;
use Drupal\Core\Form\FormState;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Language\LanguageManagerInterface;
use Drupal\Core\ParamConverter\ParamConverterManager;
use Drupal\Core\ParamConverter\ParamNotConvertedException;
use Drupal\Core\Routing\RouteMatch;
use Drupal\Core\Routing\RouteProviderInterface;
use Drupal\Core\Url;
use Symfony\Cmf\Component\Routing\RouteObjectInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\Routing\Exception\RouteNotFoundException;
use Symfony\Component\Routing\Route;

/**
 * Views area plugin allowing users to add links using route info.
 *
 * @ingroup views_area_handlers
 *
 * @ViewsArea("views_area_link_route_link")
 */
class RouteLink extends LinkBase {

  /**
   * The route provider.
   *
   * @var \Drupal\Core\Routing\RouteProviderInterface
   */
  protected $routeProvider;

  /**
   * The parameter converter manager.
   *
   * @var \Drupal\Core\ParamConverter\ParamConverterManager
   */
  protected $paramConverterManager;

  /**
   * Constructs a new Entity instance.
   *
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin_id for the plugin instance.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\Core\Language\LanguageManagerInterface $language_manager
   *   The language manager.
   * @param \Drupal\Core\Access\AccessManagerInterface $access_manager
   *   The access manager.
   * @param \Drupal\Core\Routing\RouteProviderInterface $route_provider
   *   The route provider.
   * @param \Drupal\Core\ParamConverter\ParamConverterManager $paramconverter_manager
   *   The parameter converter manager needed for validating routes.
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, LanguageManagerInterface $language_manager, AccessManagerInterface $access_manager, RouteProviderInterface $route_provider, ParamConverterManager $paramconverter_manager) {
    parent::__construct($configuration, $plugin_id, $plugin_definition, $language_manager, $access_manager);
    $this->routeProvider = $route_provider;
    $this->paramConverterManager = $paramconverter_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('language_manager'),
      $container->get('access_manager'),
      $container->get('router.route_provider'),
      $container->get('paramconverter_manager')
    );
  }

  /**
   * {@inheritdoc}
   */
  protected function defineOptions() {
    $options = parent::defineOptions();

    $options['route_info'] = [
      'default' => [
        'route_name' => '',
        'route_parameters' => [],
        'skip_param_validation' => FALSE,
        'route_options' => [
          'attributes' => [],
          'https' => 2,
        ],
      ],
    ];

    return $options;
  }

  /**
   * {@inheritdoc}
   */
  public function buildOptionsForm(&$form, FormStateInterface $form_state) {
    parent::buildOptionsForm($form, $form_state);

    $route_info = $this->options['route_info'];

    $form['route_info'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Route information'),
      '#tree' => TRUE,
    ];
    $form['route_info']['route_name'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Route'),
      '#default_value' => $route_info['route_name'],
      '#required' => TRUE,
    ];
    $form['route_info']['route_parameters'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Route parameters'),
      '#default_value' => $this->yamlEncode($route_info['route_parameters']),
      '#description' => $this->t('An associative array of route parameter names and values in YAML format. Allows tokens, but validation must be skipped to use them..'),
    ];
    $form['route_info']['skip_param_validation'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Skip parameter validation'),
      '#default_value' => $route_info['skip_param_validation'],
      '#description' => $this->t("This is useful when adding route parameters that won't validate properly, perhaps because it uses token replacements. It is recommended to test that the route validates with known working parameters before disabling validation."),
    ];
    $form['route_info']['route_options'] = [];
    $form['route_info']['route_options']['attributes'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Attributes'),
      '#default_value' => $this->yamlEncode($route_info['route_options']['attributes']),
      '#description' => $this->t('An associative YAML array of HTML attributes to add to the link. Tokens allowed.'),
    ];
    $form['route_info']['route_options']['https'] = [
      '#type' => 'radios',
      '#title' => $this->t('Protocol scheme'),
      '#options' => [
        $this->t('HTTP'),
        $this->t('HTTPS'),
        $this->t('Current scheme'),
      ],
      '#description' => $this->t('Whether this URL should point to a secure location.'),
      '#default_value' => $route_info['route_options']['https'],
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function validateOptionsForm(&$form, FormStateInterface $form_state) {
    parent::validateOptionsForm($form, $form_state);

    $options = $form_state->getValue('options');

    $route = NULL;
    try {
      $route = $this->routeProvider->getRouteByName($options['route_info']['route_name']);
    }
    catch (RouteNotFoundException $e) {
      $form_state->setError($form['route_info']['route_name'], $this->t('Route does not exist.', [
        '@exception' => $e->getMessage(),
      ]));
    }

    $route_parameters = $this->decodeYamlOpt(
      $form['route_info']['route_parameters'],
      $form_state,
      $options['route_info']['route_parameters']
    );
    $options['route_info']['route_parameters'] = $route_parameters;

    if (is_object($route) && !$options['route_info']['skip_param_validation']) {
      $this->validateRouteParameters(
        $form['route_info']['route_parameters'],
        $form_state,
        $route,
        $options['route_info']['route_name'],
        $route_parameters
      );
    }

    $options['route_info']['route_options']['attributes'] = $this->decodeYamlOpt(
      $form['route_info']['route_options']['attributes'],
      $form_state,
      $options['route_info']['route_options']['attributes']
    );

    $form_state->setValue('options', $options);
  }

  /**
   * Validates the parameters entered for the route.
   *
   * @param array $element
   *   The form element, for setError usage.
   * @param \Drupal\Core\Form\FormState $form_state
   *   The current form state.
   * @param \Symfony\Component\Routing\Route $route
   *   The Route object that the user specified.
   * @param string $route_name
   *   The name of the route.
   * @param array $route_parameters
   *   The parameters array for the route, converted from YAML users input.
   */
  public function validateRouteParameters(array $element, FormState $form_state, Route $route, $route_name, array $route_parameters) {
    $path_variables = $route->compile()->getPathVariables();

    if (count($route_parameters) > count($path_variables)) {
      $form_state->setError($element, $this->t('More parameters inputed than what was needed for the route.'));
    }

    if (empty($path_variables)) {
      return;
    }

    try {
      // ParamConverterManager relies on the route name and object being
      // available from the parameters array.
      $route_parameters[RouteObjectInterface::ROUTE_OBJECT] = $route;
      $route_parameters[RouteObjectInterface::ROUTE_NAME] = $route_name;
      $upcasted_parameters = $this->paramConverterManager->convert($route_parameters + $route->getDefaults());
      $route_match = new RouteMatch($route_name, $route, $upcasted_parameters, $route_parameters);
      $parameter_bag = $route_match->getParameters();

      foreach ($path_variables as $parameter) {
        if (!$parameter_bag->has($parameter)) {
          $form_state->setError($element, $this->t('A required parameter for the route is missing.'));
        }
      }
    }
    catch (ParamNotConvertedException $e) {
      // Uncacheable because conversion of the parameter may not have been
      // possible due to dynamic circumstances.
      $form_state->setError($element, $this->t('A parameter did not validate: @exception.', [
        '@exception' => $e->getMessage(),
      ]));
    }
  }

  /**
   * {@inheritdoc}
   */
  public function render($empty = FALSE) {
    if ($empty && empty($this->options['empty'])) {
      return [];
    }

    // The form values for this element match watch \Drupal\Core\Url expects,
    // besides '2' being using the current scheme.
    if ($this->options['route_info']['route_options']['https'] == 2) {
      unset($this->options['route_info']['route_options']['https']);
    }

    $url = Url::fromRoute(
      $this->options['route_info']['route_name'],
      $this->options['route_info']['route_parameters'],
      $this->options['route_info']['route_options']
    );

    return $this->renderUrl($url);
  }

}
