<?php

namespace Drupal\views_area_link\Plugin\views\area;

use Drupal\Component\Utility\Html;
use Drupal\Component\Utility\UrlHelper;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Url;

/**
 * Views area RoutingLink handler.
 *
 * @ingroup views_area_handlers
 *
 * @ViewsArea("views_area_link_url_link")
 */
class UrlLink extends LinkBase {

  /**
   * {@inheritdoc}
   */
  protected function defineOptions() {
    $options = parent::defineOptions();
    $options['url_info'] = [
      'default' => [
        'path' => '',
        'external' => FALSE,
        'replace_spaces' => FALSE,
        'path_case' => 'none',
        'alt' => '',
        'rel' => '',
        'link_class' => '',
        'target' => '',
      ],
    ];
    return $options;
  }

  /**
   * {@inheritdoc}
   */
  public function buildOptionsForm(&$form, FormStateInterface $form_state) {
    parent::buildOptionsForm($form, $form_state);

    $form['url_info'] = ['#tree' => TRUE];
    $form['url_info']['path'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Link path'),
      '#default_value' => $this->options['url_info']['path'],
      '#description' => $this->t('The Drupal path or absolute URL for this link. May use tokens.'),
      '#maxlength' => 255,
    ];
    $form['url_info']['replace_spaces'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Replace spaces with dashes'),
      '#default_value' => $this->options['url_info']['replace_spaces'],
    ];
    $form['url_info']['external'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('External server URL'),
      '#default_value' => $this->options['url_info']['external'],
      '#description' => $this->t("Links to an external server using a full URL: e.g. 'http://www.example.com' or 'www.example.com'."),
    ];
    $form['url_info']['path_case'] = [
      '#type' => 'select',
      '#title' => $this->t('Transform the case'),
      '#description' => $this->t('When printing URL paths, how to transform the case of the filter value.'),
      '#options' => [
        'none' => $this->t('No transform'),
        'upper' => $this->t('Upper case'),
        'lower' => $this->t('Lower case'),
        'ucfirst' => $this->t('Capitalize first letter'),
        'ucwords' => $this->t('Capitalize each word'),
      ],
      '#default_value' => $this->options['url_info']['path_case'],
    ];
    $form['url_info']['link_class'] = [
      '#title' => $this->t('Link class'),
      '#type' => 'textfield',
      '#default_value' => $this->options['url_info']['link_class'],
      '#description' => $this->t('The CSS class to apply to the link. May use tokens.'),
    ];
    $form['url_info']['alt'] = [
      '#title' => $this->t('Title text'),
      '#type' => 'textfield',
      '#default_value' => $this->options['url_info']['alt'],
      '#description' => $this->t('Text to place as "title" text which most browsers display as a tooltip when hovering over the link. May use tokens.'),
    ];
    $form['url_info']['rel'] = [
      '#title' => $this->t('Rel Text'),
      '#type' => 'textfield',
      '#default_value' => $this->options['url_info']['rel'],
      '#description' => $this->t('Include Rel attribute for use in lightbox2 or other javascript utility. May use tokens.'),
    ];
    $form['url_info']['target'] = [
      '#title' => $this->t('Target'),
      '#type' => 'textfield',
      '#default_value' => $this->options['url_info']['target'],
      '#description' => $this->t("Target of the link, such as _blank, _parent or an iframe's name. This field is rarely used. May use tokens."),
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function render($empty = FALSE) {
    // Note: Method adapted from the renderAsLinkmethod from
    // Drupal\views\Plugin\views\field\FieldPluginBase.
    if ($empty && empty($this->options['empty'])) {
      return [];
    }

    $url_info = $this->options['url_info'];
    $text = $this->options['base_info']['link_text'];
    $options = $this->options;

    $path = $url_info['path'];
    if ($path != '<front>') {
      // Use strip_tags as there should never be HTML in the path.
      // However, we need to preserve special characters like " that were
      // removed by SafeMarkup::checkPlain().
      $path = Html::decodeEntities($this->tokenizeValue($path));

      // Tokens might contain <front>, so check for <front> again.
      if ($path != '<front>') {
        $path = strip_tags($path);
      }

      // Tokens might have resolved URL's, as is the case for tokens provided by
      // Link fields, so all internal paths will be prefixed by base_path(). For
      // proper further handling reset this to internal:/.
      if (strpos($path, base_path()) === 0) {
        $path = 'internal:/' . substr($path, strlen(base_path()));
      }

      // If we have no $path and no $url_info['url'], we have nothing to work
      // with, so we just return the text.
      if (empty($path) && empty($url_info['url'])) {
        return $text;
      }

      // If no scheme is provided in the $path, assign the default 'http://'.
      // This allows a url of 'www.example.com' to be converted to
      // 'http://www.example.com'.
      // Only do this when flag for external has been set, $path doesn't contain
      // a scheme and $path doesn't have a leading /.
      if ($url_info['external'] && !parse_url($path, PHP_URL_SCHEME) && strpos($path, '/') !== 0) {
        // There is no scheme, add the default 'http://' to the $path.
        $path = "http://" . $path;
      }
    }

    if (!parse_url($path, PHP_URL_SCHEME)) {
      $url = Url::fromUserInput('/' . ltrim($path, '/'));
    }
    else {
      $url = Url::fromUri($path);
    }

    $path = $url->setOptions($options)->toUriString();

    if (!empty($url_info['path_case']) && $url_info['path_case'] != 'none' && !$url_info['url']->isRouted()) {
      $path = str_replace($url_info['path'], $this->caseTransform($url_info['path'], $this->options['url_info']['path_case']), $path);
    }

    if (!empty($url_info['replace_spaces'])) {
      $path = str_replace(' ', '-', $path);
    }

    // Parse the URL and move any query and fragment parameters out of the path.
    $url = UrlHelper::parse($path);

    // Seriously malformed URLs may return FALSE or empty arrays.
    if (empty($url)) {
      return $text;
    }

    // If the path is empty do not build a link around the given text and return
    // it as is. http://www.example.com URLs will not have a $url['path'], so
    // check host as well.
    if (empty($url['path']) && empty($url['host']) && empty($url['fragment']) && empty($url['url'])) {
      return $text;
    }

    // If we get to here we have a path from the url parsing. So assign that to
    // $path now so we don't get query strings or fragments in the path.
    $path = $url['path'];

    if (isset($url['query'])) {
      // Remove query parameters that were assigned a query string replacement
      // token for which there is no value available.
      foreach ($url['query'] as $param => $val) {
        if ($val == '%' . $param) {
          unset($url['query'][$param]);
        }
        // Replace any empty query params from URL parsing with NULL. So the
        // query will get built correctly with only the param key.
        // @see \Drupal\Component\Utility\UrlHelper::buildQuery().
        if ($val === '') {
          $url['query'][$param] = NULL;
        }
      }

      $options['query'] = $url['query'];
    }

    if (isset($url['fragment'])) {
      $path = strtr($path, ['#' . $url['fragment'] => '']);
      // If the path is empty we want to have a fragment for the current site.
      if ($path == '') {
        $options['external'] = TRUE;
      }
      $options['fragment'] = $url['fragment'];
    }

    $alt = $this->tokenizeValue($url_info['alt']);
    // Set the title attribute of the link only if it improves accessibility.
    if ($alt && $alt != $text) {
      $options['attributes']['title'] = Html::decodeEntities($alt);
    }

    $class = $this->tokenizeValue($url_info['link_class']);
    if ($class) {
      $options['attributes']['class'] = [$class];
    }

    if (!empty($url_info['rel']) && $rel = $this->tokenizeValue($url_info['rel'])) {
      $options['attributes']['rel'] = $rel;
    }

    $target = trim($this->tokenizeValue($url_info['target']));
    if (!empty($target)) {
      $options['attributes']['target'] = $target;
    }

    if (isset($url_info['link_attributes']) && is_array($url_info['link_attributes'])) {
      foreach ($url_info['link_attributes'] as $key => $attribute) {
        if (!isset($options['attributes'][$key])) {
          $options['attributes'][$key] = $this->tokenizeValue($attribute);
        }
      }
    }

    // If the query and fragment were programmatically assigned overwrite any
    // parsed values.
    if (isset($url_info['query'])) {
      // Convert the query to a string, perform token replacement, and then
      // convert back to an array form for
      // \Drupal\Core\Utility\LinkGeneratorInterface::generate().
      $options['query'] = UrlHelper::buildQuery($url_info['query']);
      $options['query'] = $this->tokenizeValue($options['query']);
      $query = [];
      parse_str($options['query'], $query);
      $options['query'] = $query;
    }
    if (isset($url_info['alias'])) {
      // Alias is a boolean field, so no token.
      $options['alias'] = $url_info['alias'];
    }
    if (isset($url_info['fragment'])) {
      $options['fragment'] = $this->tokenizeValue($url_info['fragment']);
    }

    return $this->renderUrl(Url::fromUri($path, $options));
  }

}
